package edu.harvard.mgh.purcell.gplink.mainGUI;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;
import java.util.Vector;

import javax.swing.JMenuItem;


import edu.harvard.mgh.purcell.gplink.data.Project;
import edu.harvard.mgh.purcell.gplink.base_form.GeneralUtil;
/**
 * A simple popup for editing or deleting operations.
 * @author Kathe Todd-Brown
 */
@SuppressWarnings("serial")
public class OpTreePopup extends GplinkPopUpMenu {

	
	Project data;
	String opName;
	MainFrame frame;
	String filename;
	String remoteFilename;
	String selectedFilename;
	
	/**
	 * call an operation editing dialog
	 */
	private ActionListener exitOp = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new EditOpDialog(opName, frame);
		}
	};
	
	/**
	 * create a new operation based on opName
	 */
	private ActionListener createOp = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new PlinkExicute(frame, data.getOpCommand(opName));
		}
	};
	
	/**
	 * delete an operation
	 */
	private ActionListener deleteOp = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new DeleteOpDialog(frame, new String [] { opName});
		}
	};
	private ActionListener unlinkFile = new ActionListener(){

		public void actionPerformed(ActionEvent arg0) {
			//TODO add yes/no dialog
			Vector <String> operations = new Vector <String>();
			Vector <String> filenames = new Vector <String>();
			
			operations.add(opName);
			filenames.add(selectedFilename);
			
			data.removeFile(operations, filenames);
			data.frame.updateViews();
		}
		
	};
	
	private ActionListener linkFile = new ActionListener(){

		public void actionPerformed(ActionEvent arg0) {
			new LinkNewFileDialog(frame, opName);
			
		}
		
	};
	
	/**
	 * Create a simple popup attached to the main frame and pass in the 
	 * needed data.
	 * @param givenFrame the main frame that the popup is attached to
	 * @param givenData the project that holds the data for all operations
	 * @param givenOpName the name of the operation we are editing
	 */
	public OpTreePopup(MainFrame givenFrame, Project givenData, String [] givenOpName,
			String [] filename){
		super(givenFrame, filename, givenOpName);
		
		opName = givenOpName[0];
		data = givenData;
		frame = givenFrame;
		
		//point the filename to the local copy of the file
		if(data.isRemote(false) == true){
			if(filename != null){
			remoteFilename = filename[0];
			File remoteCopy = new File(data.getRoot(), GeneralUtil.fileName(filename[0]));
			filename[0] = remoteCopy.getAbsolutePath();
			}
			else 
				remoteFilename = null;
		}
		
		/*
		 * create the menu items
		 */
		JMenuItem edit = new JMenuItem("Edit");
		JMenuItem create = new JMenuItem("Create New");
		JMenuItem link = new JMenuItem("Link new input/output file");
		JMenuItem delete = new JMenuItem("Delete");
		JMenuItem unlink = new JMenuItem("Unlink from operation");
			
		edit.addActionListener(exitOp);
		create.addActionListener(createOp);
		delete.addActionListener(deleteOp);
		unlink.addActionListener(unlinkFile);
		link.addActionListener(linkFile);
		
		/*
		 * Populate the menu
		 */

		if(filename != null
				&& data.getBrowse() == false){
			add(unlink);
		}
		
		if(data.hasMetaLock() == true){
			add(link);
			add(edit);
			if(data.getBrowse() == false)
				add(create);
			add(delete);
			
		}
		
		
		
		pack();
	}

}
