package edu.harvard.mgh.purcell.gplink.mainGUI;


import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
//import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
//import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.JTextComponent;

import ch.ethz.ssh2.Connection;
import ch.ethz.ssh2.Session;
import ch.ethz.ssh2.StreamGobbler;

/**
 * Create a dialog that allows for the selection of file and folder
 * from a remote unix-like system. Note that this system must accept
 * "ls" command.
 * @author Kathe Todd-Brown
 */
@SuppressWarnings("serial")
public class RemoteFileChooser extends JDialog {
	
	/**
	 * the external target that this dialog is attached to
	 */
	private JTextComponent target;
	
	/**
	 * a list of files&folders in a directory
	 */
	private JList fileList;
	
	/** 
	 * allows us to change this list after it is set
	 */
	private DefaultListModel editableList;
	
	/** 
	 * the connection to our remote machine
	 */
	private Connection conn;
	
	/**
	 * Panel that allows us to go back in the directory hierarchy
	 */
	private JPanel nav;
	
	/**
	 * Panel that displays the quit/process buttons
	 */
	private JPanel buttons;
	
	/**
	 * displays the current file/folder that is selected
	 */
	private JTextField info;
	
	/**
	 * the current directory we are in
	 */
	private String dir;
	
	/**
	 * flags if folders are ok to select
	 */
	private boolean pickDir;

	/**
	 * Create a dialog that lets us pick a file.
	 * @param f dialog that this frame is attached to.
	 * @param c connection that the ssh quiries are sent through
	 * @param t the textcompoent that we are dumping the file names to
	 * @param currentDir the directory that we are current in remotely
	 * @param selectDir flag that says directories are ok to select
	 */
	public RemoteFileChooser(Dialog f, Connection c, 
			JTextComponent t, String currentDir, 
			boolean selectDir){
		
		super(f, "Choose Remote File");
		
		fillContent(c, t, currentDir, selectDir);
		
		pack();
		setVisible(true);
	}
	
	/**
	 * Create a dialog that lets us pick a file.
	 * @param f frame that this frame is attached to.
	 * @param c connection that the ssh quiries are sent through
	 * @param t the textcompoent that we are dumping the file names to
	 * @param currentDir the directory that we are current in remotely
	 * @param selectDir flag that says directories are ok to select
	 */
	public RemoteFileChooser(Frame f, Connection c, 
			JTextComponent t, String currentDir, 
			boolean selectDir){
		
		super(f, "Choose Remote File");
		
		fillContent(c, t, currentDir, selectDir);
		
		pack();
		setVisible(true);
	}
	
	private void fillContent(Connection c, 
			JTextComponent t, String currentDir, 
			boolean selectDir){
		/*
		 * set the parameters
		 */
		conn = c;
		dir = currentDir;
		pickDir = selectDir;
		info = new JTextField(30);
		info.setEditable(false);
		editableList = new DefaultListModel();
		fileList = new JList(editableList);
		target = t;
		
		/*
		 * create the content
		 */
		createContent();
		createNav();
		createButtons();
		
		/*
		 * layout the content
		 */
		this.setLayout(new BorderLayout());
		getContentPane().add(new JScrollPane(fileList), BorderLayout.CENTER);
		getContentPane().add(nav, BorderLayout.PAGE_START);
		getContentPane().add(buttons, BorderLayout.PAGE_END);
	}
	
	/**
	 * Go into a folder with it is double clicked or select
	 * it for the target if it is a file
	 */
	private  MouseListener pickAFile =  new MouseListener(){

		public void mouseClicked(MouseEvent e) {
			//if it's a double click
			if(e.getClickCount() == 2){
				//select that file and go into a folder
				RemoteFileChooser.this.fileSelect(false);
			}
		}

		public void mouseEntered(MouseEvent e) {}

		public void mouseExited(MouseEvent e) {}

		public void mousePressed(MouseEvent e) {}

		public void mouseReleased(MouseEvent e) {}
	};
	
	/**
	 * Pass the selected file to the target text component.
	 */
	private ActionListener openAFile = new ActionListener(){
		public void actionPerformed(ActionEvent arg0) {
			//if we can't pick directories and the selected
			//...item is a directory
			if(RemoteFileChooser.this.pickDir == false
					&& RemoteFileChooser.this.selectedIsDirectory()){
				//do nothing
			}else{
				//otherwise send it to our listener
				target.setText(info.getText());
				//and close up shop
				RemoteFileChooser.this.dispose();
				}
			}
	};
	
	/**
	 * Close the dialog nicely.
	 */
	private ActionListener closeDialog = new ActionListener(){

		public void actionPerformed(ActionEvent e) {
			RemoteFileChooser.this.dispose();
		}
		
	};
	
	/**
	 * Back up to the parent directory.
	 */
	private ActionListener goToParent = new ActionListener(){
		public void actionPerformed(ActionEvent arg0) {
			//check to make sure that there is a parent directory
			if(dir.equals("/"))
				return;
			
			//cut off the last file in the directory name
			String newDir = dir.replaceFirst("/[^/]+$", "");
			//if this didn't do anything or we are at ~/
			if(newDir.equals(dir)
					||newDir.equals("~/")){
				//get the "real" full name
				newDir = getFullName();
				//and run it again
				newDir = newDir.replaceFirst("/[^/]+$", "");
			}
			//double check that we haven't cut off the home dir
			if(newDir.equals(""))
				newDir = "/";
			
			//reset the directory
			dir = newDir;
			//remake the list
			createContent();
		}
		
	};
	
	/**
	 * Put the file names into the list.
	 */
	private ListSelectionListener populateinfo = new ListSelectionListener(){
		public void valueChanged(ListSelectionEvent arg0) {
			String name = (String) fileList.getSelectedValue();
			if(name == null)
				return;
			if(name.endsWith("/") 
					|| name.endsWith("@")
					|| name.endsWith("*")
					|| name.endsWith("=")
					|| name.endsWith("%")
					|| name.endsWith("|"))
				name = name.substring(0, name.length()-1);
			info.setText(dir + "/" + name); 
		}
	};
	
	/**
	 * Get the full path name of the current directory
	 * @return the full path name of the current diectory
	 */
	private String getFullName(){
		String ans = "";
		try{
			
			/* Create a session */
	
			Session sess = conn.openSession();
			
			/* get the list of files from a unix system */
			sess.execCommand("pwd " + dir);
			InputStream stdout = new StreamGobbler(sess.getStdout());
			BufferedReader br = new BufferedReader(new InputStreamReader(stdout));
			while (true)
			{
				String line = br.readLine();
				if (line == null)
					break;
				//System.out.println(line);
				ans = line;
			}
	
			// Close this session
			sess.close();
		}catch(IOException e){
			
		}
		return ans;
	}
	
	/**
	 * Get a list of files in a directory
	 * @param filename a string with a valid path name for the
	 * remote project
	 * @return a vector containing the file names of all files in
	 * the current directory
	 */
	private Vector <String> getFileContent(String filename){

		Vector <String> files = new Vector<String>();
		
		try{
			
			/* Create a session */
	
			Session sess = conn.openSession();
			
			/* get the list of files from a unix system */
			sess.execCommand("ls -FL " + filename);

			InputStream stdout = new StreamGobbler(sess.getStdout());
	
			BufferedReader br = new BufferedReader(new InputStreamReader(stdout));
	
			while (true)
			{
				String line = br.readLine();
				if (line == null)
					break;
				files.add(line);
			}
	
			/* Close this session */
	
			sess.close();
		}catch(IOException e){
			
		}
		return files;
	}
	
	/**
	 * 
	 * @param selectDir
	 */
	private void fileSelect( boolean selectDir){
		//String given = (String) fileList.getSelectedValue();
		if(selectedIsDirectory() && selectDir == false){
			dir = info.getText();
			createContent();
		}
	}
	
	public boolean selectedIsDirectory(){
		if(fileList.getSelectedValue() == null)
			return false;
		boolean ans = ((String) fileList.getSelectedValue()).contains("/");
		
		return ans;
	}
	
	private void createNav(){
		nav = new JPanel();
		
		JButton up = new JButton("Back");
		up.addActionListener(goToParent);
		nav.add(up);
	}
	
	private void createButtons(){
		buttons = new JPanel();
		
		JButton ok = new JButton("Open");
		ok.addActionListener(openAFile);
		JButton cancel = new JButton("Cancel");
		cancel.addActionListener(closeDialog);
		
		buttons.add(info);
		buttons.add(ok);
		buttons.add(cancel);
		
	}
	
	private void createContent(){
		Vector <String> files = getFileContent(dir);
		fileList.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		fileList.setLayoutOrientation(JList.VERTICAL);
		fileList.addMouseListener(pickAFile);
		fileList.addListSelectionListener(populateinfo);
		editableList.removeAllElements();
		for(String current: files){
			//puting a filter on here will slow this process down
			//...too much
			editableList.addElement(current);
		}
		info.setText(dir);
		fileList.ensureIndexIsVisible(0);
	}
	
	
	
	

}
